import os
import json
import time
from datetime import datetime, timedelta


class ScheduleManager:
    def __init__(self):
        self.schedules_dir = os.path.join(os.getcwd(), "schedules")
        self.schedules_file = os.path.join(
            self.schedules_dir, "schedules.json"
        )
        self.schedule_types = [
            "meeting", "appointment", "event", "deadline", "reminder"
        ]
        self.schedule_statuses = [
            "planned", "in_progress", "completed", "canceled", "postponed"
        ]

        # 确保目录存在
        os.makedirs(self.schedules_dir, exist_ok=True)

        # 初始化数据
        self.schedules = self._load_schedules()
    
    def _load_schedules(self):
        """加载日程数据"""
        if os.path.exists(self.schedules_file):
            with open(self.schedules_file, "r", encoding="utf-8") as f:
                return json.load(f)
        return []
    
    def _save_schedules(self):
        """保存日程数据"""
        with open(self.schedules_file, "w", encoding="utf-8") as f:
            json.dump(self.schedules, f, ensure_ascii=False, indent=2)
    
    def create_schedule(self, schedule_data):
        """创建日程"""
        # 生成唯一ID
        schedule_id = f"schedule_{int(time.time())}"
        
        # 验证类型和状态
        schedule_type = schedule_data.get("type", "meeting")
        if schedule_type not in self.schedule_types:
            schedule_type = "meeting"
        
        status = schedule_data.get("status", "planned")
        if status not in self.schedule_statuses:
            status = "planned"
        
        # 创建日程对象
        schedule = {
            "id": schedule_id,
            "title": schedule_data.get("title", "新建日程"),
            "description": schedule_data.get("description", ""),
            "type": schedule_type,
            "status": status,
            "start_time": schedule_data.get("start_time", datetime.now().strftime("%Y-%m-%d %H:%M:%S")),
            "end_time": schedule_data.get("end_time", datetime.now().strftime("%Y-%m-%d %H:%M:%S")),
            "location": schedule_data.get("location", ""),
            "organizer": schedule_data.get("organizer", ""),
            "attendees": schedule_data.get("attendees", []),
            "reminders": schedule_data.get("reminders", []),
            "repeats": schedule_data.get("repeats", False),
            "repeat_type": schedule_data.get("repeat_type", "none"),  # none, daily, weekly, monthly, yearly
            "repeat_end_date": schedule_data.get("repeat_end_date", ""),
            "tags": schedule_data.get("tags", []),
            "links": schedule_data.get("links", []),
            "attachments": schedule_data.get("attachments", []),
            "created_at": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            "updated_at": datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        }
        
        # 添加到日程列表
        self.schedules.append(schedule)
        self._save_schedules()
        
        return schedule
    
    def get_schedules(self, filters=None):
        """获取日程列表，支持筛选"""
        if not filters:
            return self.schedules
        
        filtered_schedules = self.schedules
        
        # 类型筛选
        if "type" in filters:
            filtered_schedules = [s for s in filtered_schedules if s["type"] == filters["type"]]
        
        # 状态筛选
        if "status" in filters:
            filtered_schedules = [s for s in filtered_schedules if s["status"] == filters["status"]]
        
        # 组织者筛选
        if "organizer" in filters:
            filtered_schedules = [s for s in filtered_schedules if s["organizer"] == filters["organizer"]]
        
        # 参会人员筛选
        if "attendee" in filters:
            filtered_schedules = [s for s in filtered_schedules if filters["attendee"] in s["attendees"]]
        
        # 日期范围筛选
        if "start_date" in filters:
            filtered_schedules = [s for s in filtered_schedules if s["start_time"] >= filters["start_date"]]
        
        if "end_date" in filters:
            filtered_schedules = [s for s in filtered_schedules if s["end_time"] <= filters["end_date"]]
        
        return filtered_schedules
    
    def get_schedule(self, schedule_id):
        """获取指定日程"""
        return next((s for s in self.schedules if s["id"] == schedule_id), None)
    
    def update_schedule(self, schedule_id, schedule_data):
        """更新日程"""
        for i, schedule in enumerate(self.schedules):
            if schedule["id"] == schedule_id:
                # 验证类型和状态
                if "type" in schedule_data:
                    if schedule_data["type"] not in self.schedule_types:
                        del schedule_data["type"]
                
                if "status" in schedule_data:
                    if schedule_data["status"] not in self.schedule_statuses:
                        del schedule_data["status"]
                
                # 更新日程数据
                self.schedules[i].update(schedule_data)
                self.schedules[i]["updated_at"] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                self._save_schedules()
                return self.schedules[i]
        return None
    
    def delete_schedule(self, schedule_id):
        """删除日程"""
        self.schedules = [s for s in self.schedules if s["id"] != schedule_id]
        self._save_schedules()
        return True
    
    def add_attendee(self, schedule_id, attendee):
        """添加参会人员"""
        schedule = self.get_schedule(schedule_id)
        if not schedule:
            return None
        
        # 检查参会人员是否已存在
        if attendee not in schedule["attendees"]:
            schedule["attendees"].append(attendee)
            return self.update_schedule(schedule_id, schedule)
        
        return schedule
    
    def remove_attendee(self, schedule_id, attendee):
        """移除参会人员"""
        schedule = self.get_schedule(schedule_id)
        if not schedule:
            return None
        
        if attendee in schedule["attendees"]:
            schedule["attendees"].remove(attendee)
            return self.update_schedule(schedule_id, schedule)
        
        return schedule
    
    def add_reminder(self, schedule_id, reminder):
        """添加提醒"""
        schedule = self.get_schedule(schedule_id)
        if not schedule:
            return None
        
        # 生成提醒ID
        reminder_id = f"reminder_{int(time.time())}"
        
        # 创建提醒对象
        new_reminder = {
            "id": reminder_id,
            "time": reminder.get("time", ""),
            "method": reminder.get("method", "email"),  # email, sms, push
            "created_at": datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        }
        
        # 初始化reminders字段（如果不存在）
        if "reminders" not in schedule:
            schedule["reminders"] = []
        
        # 添加提醒到日程
        schedule["reminders"].append(new_reminder)
        return self.update_schedule(schedule_id, schedule)
    
    def get_upcoming_schedules(self, days=7):
        """获取未来几天的日程"""
        today = datetime.now()
        future_date = today + timedelta(days=days)
        
        upcoming_schedules = []
        for schedule in self.schedules:
            try:
                start_time = datetime.strptime(schedule["start_time"], "%Y-%m-%d %H:%M:%S")
                if today <= start_time <= future_date and schedule["status"] != "completed" and schedule["status"] != "canceled":
                    upcoming_schedules.append(schedule)
            except:
                continue
        
        # 按开始时间排序
        upcoming_schedules.sort(key=lambda x: x["start_time"])
        return upcoming_schedules
    
    def get_todays_schedules(self):
        """获取今天的日程"""
        today = datetime.now().strftime("%Y-%m-%d")
        return [s for s in self.schedules if s["start_time"].startswith(today) and s["status"] != "completed" and s["status"] != "canceled"]
    
    def check_schedule_conflict(self, start_time, end_time, exclude_schedule_id=None):
        """检查日程冲突"""
        """
        检查给定时间范围内是否存在日程冲突
        
        Args:
            start_time (str): 开始时间，格式为"%Y-%m-%d %H:%M:%S"
            end_time (str): 结束时间，格式为"%Y-%m-%d %H:%M:%S"
            exclude_schedule_id (str, optional): 要排除的日程ID，用于更新日程时避免与自身冲突
        
        Returns:
            list: 冲突的日程列表
        """
        conflicts = []
        
        try:
            new_start = datetime.strptime(start_time, "%Y-%m-%d %H:%M:%S")
            new_end = datetime.strptime(end_time, "%Y-%m-%d %H:%M:%S")
        except ValueError:
            # 时间格式错误，返回空列表
            return conflicts
        
        for schedule in self.schedules:
            # 跳过要排除的日程
            if schedule["id"] == exclude_schedule_id:
                continue
            
            try:
                schedule_start = datetime.strptime(schedule["start_time"], "%Y-%m-%d %H:%M:%S")
                schedule_end = datetime.strptime(schedule["end_time"], "%Y-%m-%d %H:%M:%S")
            except ValueError:
                continue
            
            # 检查时间冲突
            if (new_start < schedule_end and new_end > schedule_start):
                conflicts.append(schedule)
        
        return conflicts
    
    def get_schedule_statistics(self):
        """获取日程统计信息"""
        total_schedules = len(self.schedules)
        
        # 按类型统计
        type_stats = {}
        for schedule_type in self.schedule_types:
            type_stats[schedule_type] = len([s for s in self.schedules if s["type"] == schedule_type])
        
        # 按状态统计
        status_stats = {}
        for status in self.schedule_statuses:
            status_stats[status] = len([s for s in self.schedules if s["status"] == status])
        
        # 今日日程数
        todays_schedules = len(self.get_todays_schedules())
        
        # 即将到来的日程数（未来7天）
        upcoming_schedules = len(self.get_upcoming_schedules(7))
        
        return {
            "total_schedules": total_schedules,
            "type_stats": type_stats,
            "status_stats": status_stats,
            "todays_schedules": todays_schedules,
            "upcoming_schedules": upcoming_schedules
        }
    
    def add_attachment(self, schedule_id, attachment_data):
        """添加日程附件"""
        schedule = self.get_schedule(schedule_id)
        if not schedule:
            return None
        
        # 生成附件ID
        attachment_id = f"attachment_{int(time.time())}"
        
        # 创建附件对象
        attachment = {
            "id": attachment_id,
            "name": attachment_data.get("name", ""),
            "file_path": attachment_data.get("file_path", ""),
            "file_type": attachment_data.get("file_type", ""),
            "file_size": attachment_data.get("file_size", 0),
            "uploaded_by": attachment_data.get("uploaded_by", ""),
            "uploaded_at": datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        }
        
        # 初始化attachments字段（如果不存在）
        if "attachments" not in schedule:
            schedule["attachments"] = []
        
        # 添加附件到日程
        schedule["attachments"].append(attachment)
        return self.update_schedule(schedule_id, schedule)
    
    def remove_attachment(self, schedule_id, attachment_id):
        """移除日程附件"""
        schedule = self.get_schedule(schedule_id)
        if not schedule:
            return None
        
        # 过滤掉要删除的附件
        schedule["attachments"] = [a for a in schedule.get("attachments", []) if a["id"] != attachment_id]
        return self.update_schedule(schedule_id, schedule)


from docx import Document
from docx.shared import RGBColor
import os
import json
from datetime import datetime

class DocxTemplateParser:
    """
    DOCX模板解析器，用于解析DOCX文件结构并生成模板配置
    """
    
    def __init__(self):
        pass
    
    def parse_docx(self, file_path):
        """
        解析DOCX文件，提取结构信息
        
        Args:
            file_path: DOCX文件路径
            
        Returns:
            dict: 包含模板结构信息的字典
        """
        try:
            # 加载DOCX文档
            doc = Document(file_path)
            
            # 解析文档结构
            structure = {
                'tables': self._parse_tables(doc),
                'images': self._parse_images(doc),
                'paragraphs': self._parse_paragraphs(doc),
                'placeholders': self._extract_placeholders(doc)
            }
            
            # 提取纯文本内容
            text_content = self._extract_text_content(doc)
            
            return {
                'success': True,
                'content': text_content,
                'structure': structure,
                'message': 'DOCX文件解析成功'
            }
        except Exception as e:
            return {
                'success': False,
                'message': f'解析DOCX文件失败: {str(e)}'
            }
    
    def _parse_tables(self, doc):
        """解析表格结构"""
        tables = []
        
        for table_idx, table in enumerate(doc.tables):
            table_data = {
                'table_id': f'table_{table_idx}',
                'rows': len(table.rows),
                'cols': len(table.columns),
                'cells': [],
                'style': self._get_table_style(table)
            }
            
            # 遍历表格单元格
            for row_idx, row in enumerate(table.rows):
                for col_idx, cell in enumerate(row.cells):
                    # 检查是否合并单元格
                    is_merged = False
                    if row_idx > 0 or col_idx > 0:
                        # 简单合并检测（实际实现可能需要更复杂的逻辑）
                        if cell._tc.top != row_idx or cell._tc.left != col_idx:
                            is_merged = True
                    
                    cell_data = {
                        'row': row_idx,
                        'col': col_idx,
                        'text': cell.text.strip(),
                        'is_merged': is_merged,
                        'format': self._get_cell_format(cell)
                    }
                    table_data['cells'].append(cell_data)
            
            tables.append(table_data)
        
        return tables
    
    def _parse_images(self, doc):
        """解析图片位置和格式"""
        images = []
        image_idx = 0
        
        for paragraph_idx, paragraph in enumerate(doc.paragraphs):
            for run_idx, run in enumerate(paragraph.runs):
                for inline_shape in run._inline_shapes:
                    if inline_shape.type == 3:  # 图片类型
                        image_data = {
                            'image_id': f'image_{image_idx}',
                            'paragraph_idx': paragraph_idx,
                            'run_idx': run_idx,
                            'width': inline_shape.width.inches,
                            'height': inline_shape.height.inches,
                            'format': self._get_image_format(inline_shape)
                        }
                        images.append(image_data)
                        image_idx += 1
        
        return images
    
    def _parse_paragraphs(self, doc):
        """解析段落样式"""
        paragraphs = []
        
        for para_idx, paragraph in enumerate(doc.paragraphs):
            # 检查段落类型
            is_title = self._is_title(paragraph)
            is_list = self._is_list(paragraph)
            
            para_data = {
                'para_id': f'para_{para_idx}',
                'text': paragraph.text.strip(),
                'is_title': is_title,
                'is_list': is_list,
                'heading_level': self._get_heading_level(paragraph),
                'format': self._get_paragraph_format(paragraph),
                'runs': self._parse_runs(paragraph)
            }
            paragraphs.append(para_data)
        
        return paragraphs
    
    def _parse_runs(self, paragraph):
        """解析段落中的文本运行（runs）"""
        runs = []
        
        for run_idx, run in enumerate(paragraph.runs):
            run_data = {
                'run_id': f'run_{run_idx}',
                'text': run.text,
                'format': self._get_run_format(run)
            }
            runs.append(run_data)
        
        return runs
    
    def _extract_placeholders(self, doc):
        """提取模板中的占位符，如{{content}}"""
        placeholders = []
        placeholder_set = set()
        
        # 从段落中提取
        for paragraph in doc.paragraphs:
            text = paragraph.text
            self._find_placeholders_in_text(text, placeholder_set)
        
        # 从表格中提取
        for table in doc.tables:
            for row in table.rows:
                for cell in row.cells:
                    text = cell.text
                    self._find_placeholders_in_text(text, placeholder_set)
        
        # 转换为列表
        for placeholder in placeholder_set:
            placeholders.append({
                'name': placeholder,
                'placeholder': f'{{{{{placeholder}}}}}'
            })
        
        return placeholders
    
    def _find_placeholders_in_text(self, text, placeholder_set):
        """在文本中查找占位符"""
        import re
        # 匹配{{placeholder}}格式的占位符
        matches = re.findall(r'{{(\w+)}}', text)
        for match in matches:
            placeholder_set.add(match.strip())
    
    def _extract_text_content(self, doc):
        """提取文档的纯文本内容"""
        text_content = []
        
        for paragraph in doc.paragraphs:
            text_content.append(paragraph.text)
        
        return '\n'.join(text_content)
    
    # 格式获取辅助方法
    
    def _get_table_style(self, table):
        """获取表格样式"""
        return {
            'style_name': table.style.name if table.style else 'Default'
        }
    
    def _get_cell_format(self, cell):
        """获取单元格格式"""
        # 简化实现，仅获取第一个段落的格式
        if cell.paragraphs:
            para_format = self._get_paragraph_format(cell.paragraphs[0])
            return para_format
        return {}
    
    def _get_paragraph_format(self, paragraph):
        """获取段落格式"""
        format = {
            'alignment': str(paragraph.alignment),
            'space_before': paragraph.paragraph_format.space_before.pt if paragraph.paragraph_format.space_before else 0,
            'space_after': paragraph.paragraph_format.space_after.pt if paragraph.paragraph_format.space_after else 0,
            'line_spacing': paragraph.paragraph_format.line_spacing
        }
        
        # 获取首行缩进
        if paragraph.paragraph_format.first_line_indent:
            format['first_line_indent'] = paragraph.paragraph_format.first_line_indent.pt
        
        return format
    
    def _get_run_format(self, run):
        """获取文本运行格式"""
        font = run.font
        format = {
            'bold': font.bold,
            'italic': font.italic,
            'underline': font.underline,
            'font_name': font.name,
            'font_size': font.size.pt if font.size else None,
            'color': self._get_color(font.color)
        }
        
        return format
    
    def _get_color(self, color):
        """获取颜色信息"""
        if color.type == 1:  # RGB颜色
            return {
                'type': 'rgb',
                'value': f'#{color.rgb[0]:02x}{color.rgb[1]:02x}{color.rgb[2]:02x}'
            }
        return {
            'type': 'default'
        }
    
    def _get_image_format(self, inline_shape):
        """获取图片格式"""
        return {
            'type': 'image',
            'shape_type': inline_shape.type
        }
    
    def _is_title(self, paragraph):
        """检查是否为标题"""
        style = paragraph.style.name
        return style.startswith('Heading') or style.startswith('标题')
    
    def _is_list(self, paragraph):
        """检查是否为列表"""
        # 简化实现，通过段落样式判断
        style = paragraph.style.name
        return 'List' in style or '列表' in style
    
    def _get_heading_level(self, paragraph):
        """获取标题级别"""
        style = paragraph.style.name
        if 'Heading' in style or '标题' in style:
            # 提取级别数字
            import re
            match = re.search(r'\d+', style)
            if match:
                return int(match.group())
        return 0
    
    def generate_template_config(self, file_path, template_name, template_type='weekly_report'):
        """
        生成完整的模板配置
        
        Args:
            file_path: DOCX文件路径
            template_name: 模板名称
            template_type: 模板类型
            
        Returns:
            dict: 完整的模板配置
        """
        parse_result = self.parse_docx(file_path)
        
        if not parse_result['success']:
            return parse_result
        
        # 生成模板配置
        template_config = {
            'id': f'template_{int(datetime.now().timestamp())}',
            'name': template_name,
            'description': f'从DOCX导入的{template_type}模板',
            'type': template_type,
            'output_format': 'docx',
            'prompt': f'请根据以下模板和提示词生成{template_type}文档，内容要详细、具体、符合实际工作情况。',
            'format_settings': {
                'font_name': '微软雅黑',
                'font_size': 12,
                'line_spacing': 1.5,
                'margin': {
                    'top': 2.54,
                    'right': 2.54,
                    'bottom': 2.54,
                    'left': 2.54
                }
            },
            'batch_settings': {
                'enabled': True,
                'delimiter': '\n',
                'max_batch_size': 10
            },
            'created_at': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'updated_at': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'is_default': False,
            'content': parse_result['content'],
            'structure': parse_result['structure']
        }
        
        return {
            'success': True,
            'template': template_config,
            'message': '模板配置生成成功'
        }
    
    def save_template_config(self, template_config, save_dir):
        """
        保存模板配置到文件
        
        Args:
            template_config: 模板配置字典
            save_dir: 保存目录
            
        Returns:
            str: 保存的文件路径
        """
        # 确保保存目录存在
        os.makedirs(save_dir, exist_ok=True)
        
        # 保存模板配置
        template_id = template_config['id']
        config_path = os.path.join(save_dir, f'{template_id}_config.json')
        
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(template_config, f, ensure_ascii=False, indent=2)
        
        return config_path
